import {
    IPersistence,
    IRead,
} from "@rocket.chat/apps-engine/definition/accessors";
import {
    RocketChatAssociationModel,
    RocketChatAssociationRecord,
} from "@rocket.chat/apps-engine/definition/metadata";

interface IUserMeta {
    externalId: string;
}

interface IGroupMeta {
    externalId: string;
}

export class Store {
    private read: IRead;
    private persis: IPersistence;
    constructor(read: IRead, persis: IPersistence) {
        this.read = read;
        this.persis = persis;
    }

    public deleteUser(id: string) {
        return this.persis.removeByAssociation(
            new RocketChatAssociationRecord(
                RocketChatAssociationModel.USER,
                id,
            ),
        );
    }

    public saveUser(id: string, meta: IUserMeta) {
        return this.persis.updateByAssociation(
            new RocketChatAssociationRecord(
                RocketChatAssociationModel.USER,
                id,
            ),
            meta,
            true,
        );
    }

    public async getUser(id: string): Promise<IUserMeta | undefined> {
        const list = await this.read
            .getPersistenceReader()
            .readByAssociation(
                new RocketChatAssociationRecord(
                    RocketChatAssociationModel.USER,
                    id,
                ),
            );
        if (list.length > 0) {
            return list[0] as IUserMeta;
        }
        return;
    }

    public deleteGroup(id: string) {
        return this.persis.removeByAssociation(
            new RocketChatAssociationRecord(
                RocketChatAssociationModel.ROOM,
                id,
            ),
        );
    }

    public saveGroup(id: string, meta: IGroupMeta) {
        return this.persis.updateByAssociation(
            new RocketChatAssociationRecord(
                RocketChatAssociationModel.ROOM,
                id,
            ),
            meta,
            true,
        );
    }

    public async getGroup(id: string): Promise<IGroupMeta | undefined> {
        const list = await this.read
            .getPersistenceReader()
            .readByAssociation(
                new RocketChatAssociationRecord(
                    RocketChatAssociationModel.ROOM,
                    id,
                ),
            );
        if (list.length > 0) {
            return list[0] as IGroupMeta;
        }
        return;
    }
}
